import sys
from ctypes import CDLL
from pathlib import Path

import pynvml
from gpustat.core import GPUStatCollection
from ruxit.api.base_plugin import BasePlugin
from ruxit.api.selectors import HostSelector



class NvidiaExtension(BasePlugin):
    def find_dll_path(self) -> str:
        
        current_file_path = Path(__file__).parent.resolve()
            
        paths = [
            Path("C:/windows/system32", "nvml.dll"),
            Path("C:/Program Files/NVIDIA Corporation/NVSMI", "nvml.dll"),
            Path(current_file_path, "nvml.dll")
        ]

        for path in paths:
            if path.exists():
                self.logger.info(f"Using driver from '{path}'")
                return f"{path}"

    def query(self, **kwargs) -> None:

        # Load the nvidia DLL on Windows because the lib has hardcoded paths
        if sys.platform != 'linux':
             pynvml.nvmlLib = CDLL(self.find_dll_path())
            

        for gpu in GPUStatCollection.new_query().gpus:
            identifier = f"{gpu.name} ({gpu.index})"
            self.results_builder.absolute(key="temperature", value=gpu.temperature, dimensions={"GPU": identifier}, entity_selector=HostSelector())
            self.results_builder.absolute(key="memory_used", value=gpu.memory_used, dimensions={"GPU": identifier}, entity_selector=HostSelector())
            self.results_builder.absolute(key="memory_total", value=gpu.memory_total, dimensions={"GPU": identifier}, entity_selector=HostSelector())
            self.results_builder.absolute(key="utilization", value=gpu.utilization, dimensions={"GPU": identifier}, entity_selector=HostSelector())
            self.results_builder.absolute(key="processes", value=len(gpu.processes), dimensions={"GPU": identifier}, entity_selector=HostSelector())

